/* calcola.c */

/* Direttive */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "albero.h"

#define ROW_LENGTH 256


/* Prototipi delle procedure secondarie */

void InterpretaLineaComando (int argc, char *argv[], char *file_espressioni);

/* Elabora le espressioni contenute nel file file_espressioni */
void ElaboraEspressioni (char *file_espressioni);

/* Carica l'espressione corrente dalla stringa nell'albero */
albero ConverteEspressione (char *Espressione);

/* Calcola il valore dell'espressione rappresentata dall'albero */
double CalcolaValore (albero T);

/* Stampa in notazione infissa l'espressione rappresentata dall'albero */
void StampaEspressione (albero T);

/* Calcola la profondita' dell'albero */
int Profondita (albero T);

/* Scompone la stringa Espressione nelle tre componenti s (operatore), s_op1 (operando1) e s_op2 (operando2) */
void ScomponeEspressione (char *Espressione, char *s, char *s_op1, char *s_op2);

/* Identifica l'operatore corrispondente alla stringa s */
Operatore IdentificaOperatore (char *s);


/* Programma principale */
int main (int argc, char *argv[])
{
  /* Parte dichiarativa */
  char file_espressioni[ROW_LENGTH];

  /* Parte esecutiva */
  InterpretaLineaComando(argc,argv,file_espressioni);

  /* Elabora le espressioni nel file */
  ElaboraEspressioni(file_espressioni);

  return EXIT_SUCCESS;
}


/* Definizione delle procedure secondarie */

void InterpretaLineaComando (int argc, char *argv[], char *file_espressioni)
{
  if (argc != 2)
  {
    fprintf(stderr,"Errore nella linea di comando!\n");
    fprintf(stderr,"%s [file_espressioni]\n",argv[0]);
    exit(EXIT_FAILURE);
  }

  strcpy(file_espressioni,argv[1]);
}


void ElaboraEspressioni (char *file_espressioni)
{
  FILE *fp;
  char Espressione[ROW_LENGTH];
  albero T;
  double v;
  int p;


  fp = fopen(file_espressioni,"r");
  if (fp == NULL)
  {
    fprintf(stderr,"Errore nell'apertura del file %s!\n",file_espressioni);
    exit(EXIT_FAILURE);
  }

  /* Legge il file una riga alla volta */
  while (fscanf(fp,"%[^\n]\n",Espressione) == 1)
  {
    /* Carica l'espressione corrente dalla stringa nell'albero */
    T = ConverteEspressione(Espressione);

    /* Calcola il valore dell'espressione rappresentata dall'albero */
    v = CalcolaValore(T);
    printf("valore     = %f\n",v);

    /* Stampa in notazione infissa l'espressione rappresentata dall'albero */
    StampaEspressione(T);
    printf("\n");

    /* Calcola la profondita' dell'albero */
    p = Profondita(T);
    printf("profondita = %d\n",p);

    /* Distrugge l'albero */
    distruggealbero(&T);
    printf("\n");
  }

  fclose(fp);
}


/* Carica l'espressione corrente dalla stringa nell'albero */
albero ConverteEspressione (char *Espressione)
{
  albero T, Ts, Td;
  Operando val;
  Operatore op;
  char s[ROW_LENGTH];
  char s_op1[ROW_LENGTH];
  char s_op2[ROW_LENGTH];

  /* Se l'espressione e' semplice (numero reale) */
  if (sscanf(Espressione,"%lf",&val) == 1)
  {
    /* Costruisce un albero composto da un solo nodo con il valore reale */
    T = costruiscealbero(NO_OP,val,EMPTY_TREE,EMPTY_TREE);
  }
  /* altrimenti identifica le tre parti dell'espressione composta */
  else
  {
    ScomponeEspressione(Espressione,s,s_op1,s_op2);
    //printf("%s\n%s\n%s\n",s,s_op1,s_op2);

    /* Trasforma il primo operando in un albero */
    Ts = ConverteEspressione(s_op1);

    /* Trasforma il secondo operando in un albero */
    Td = ConverteEspressione(s_op2);

    /* Identifica l'operatore */
    op = IdentificaOperatore(s);

    /* Costruisce l'albero a partire dall'operatore e dai due sottoalberi */
    T = costruiscealbero(op,NO_VAL,Ts,Td);
  }

  return T;
}


/* Scompone la stringa Espressione nelle tre componenti s (operatore), s_op1 (operando1) e s_op2 (operando2) */
void ScomponeEspressione (char *Espressione, char *s, char *s_op1, char *s_op2)
{
  int ie, is;
  int cont;

  /* Copia da Espressione a s dall'inizio fino alla prima '(' */
  for (ie = is = 0; Espressione[ie] != '('; ie++, is++)
    s[is] = Espressione[ie];
  s[is] = '\0';
  ie++;

  /* Copia da Espressione a s_op1 dal carattere successivo fino alla prima ',' non annidata */
  is = 0;
  cont = 0;
  while ( (Espressione[ie] != ',') || (cont != 0) )
  {
    if (Espressione[ie] == '(') cont++;
    if (Espressione[ie] == ')') cont--;

    s_op1[is] = Espressione[ie];
    ie++;
    is++;
  }
  s_op1[is] = '\0';
  ie++;

  /* Copia da Espressione a s_op2 dal carattere successivo fino alla prima ')' non annidata */
  is = 0;
  while ( (Espressione[ie] != ')') || (cont != 0) )
  {
    if (Espressione[ie] == '(') cont++;
    if (Espressione[ie] == ')') cont--;

    s_op2[is] = Espressione[ie];
    ie++;
    is++;
  }
  s_op2[is] = '\0';
  ie++;
}


/* Identifica l'operatore corrispondente alla stringa s */
Operatore IdentificaOperatore (char *s)
{
  if (strcmp(s,"somma") == 0)
    return '+';
  else if (strcmp(s,"diff") == 0)
    return '-';
  else if (strcmp(s,"prod") == 0)
    return '*';
  else if (strcmp(s,"div") == 0)
    return '/';
  else
  {
    fprintf(stderr,"Errore nell'identificazione dell'operatore %s!\n",s);
    exit(EXIT_FAILURE);
  }
}


/* Calcola il valore dell'espressione rappresentata dall'albero */
double CalcolaValore (albero T)
{
  Operatore op;
  Operando val, vals, vald;

  /* Se l'albero e' vuoto, restituise NOVAL */
  if (alberovuoto(T))
    return NO_VAL;
  else
  {
    leggenodo(T,radice(T),&op,&val);

    /* Se l'albero e' semplice (la radice ha operatore NO_OP),
      restituisce il valore della radice */
    if (op == NO_OP)
      return val;
    else
    {
      /* Se l'albero e' composto, valuta i due sottoalberi e applica l'operatore ai loro valori */
      vals = CalcolaValore(figliosinistro(T,radice(T)));
      vald = CalcolaValore(figliodestro(T,radice(T)));
      if (op == '+')
        return vals+vald;
      else if (op == '-')
        return vals-vald;
      else if (op == '*')
        return vals*vald;
      else if (op == '/')
        return vals/vald;
      else
      {
        fprintf(stderr,"Errore nell'espressione!\n");
        exit(EXIT_FAILURE);
      }
    }
  }

  return NO_VAL;
}


/* Stampa in notazione infissa l'espressione rappresentata dall'albero */
void StampaEspressione (albero T)
{
  Operatore op;
  Operando val;

  /* Se l'albero e' vuoto, non fa nulla */
  if (!alberovuoto(T))
  {
    /* Altrimenti, determina se e' un'espressione semplice (operando) o composta */
    leggenodo(T,radice(T),&op,&val);
    if (op == NO_OP)
      /* Se e' semplice, la stampa */
      printf(" %f ",val);
    else
    {
      /* Se e' composta, stampa fra parentesi primo operando, operatore, secondo operando */
      printf(" ( ");
      StampaEspressione(figliosinistro(T,radice(T)));
      printf(" %c ",op);
      StampaEspressione(figliodestro(T,radice(T)));
      printf(" ) ");
    }
  }
}


/* Calcola la profondita' dell'albero */
int Profondita (albero T)
{
  int ps, pd;

  /* Se l'albero e' vuoto, ha profondita' nulla */
  if (alberovuoto(T))
    return 0;
  else
  /* Altrimenti, la profondita' e' la maggiore dei due sottoalberi, incrementata di 1 */
  {
    ps = Profondita(figliosinistro(T,radice(T)));
    pd = Profondita(figliodestro(T,radice(T)));
    return 1 + ( (ps > pd) ? ps : pd);
  }
}
