/* calcola.c */

/* Direttive */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "albero.h"

#define ROW_LENGTH 256


/* Prototipi delle procedure secondarie */

void InterpretaLineaComando (int argc, char *argv[], char *file_espressioni);

/* Elabora le espressioni contenute nel file file_espressioni */
void ElaboraEspressioni (char *file_espressioni);

/* Carica l'espressione corrente dalla stringa nell'albero */
albero ConverteEspressione (char *Espressione);

/* Calcola il valore dell'espressione rappresentata dall'albero */
double CalcolaValore (albero T);

/* Stampa in notazione infissa l'espressione rappresentata dall'albero */
void StampaEspressione (albero T);

/* Calcola la profondita' dell'albero */
int Profondita (albero T);


/* Programma principale */
int main (int argc, char *argv[])
{
  /* Parte dichiarativa */
  char file_espressioni[ROW_LENGTH];

  /* Parte esecutiva */
  InterpretaLineaComando(argc,argv,file_espressioni);

  /* Elabora le espressioni nel file */
  ElaboraEspressioni(file_espressioni);

  return EXIT_SUCCESS;
}


/* Definizione delle procedure secondarie */

void InterpretaLineaComando (int argc, char *argv[], char *file_espressioni)
{
  if (argc != 2)
  {
    fprintf(stderr,"Errore nella linea di comando!\n");
    fprintf(stderr,"%s [file_espressioni]\n",argv[0]);
    exit(EXIT_FAILURE);
  }

  strcpy(file_espressioni,argv[1]);
}


void ElaboraEspressioni (char *file_espressioni)
{
  FILE *fp;
  char Espressione[ROW_LENGTH];
  albero T;
  double v;
  int p;


  fp = fopen(file_espressioni,"r");
  if (fp == NULL)
  {
    fprintf(stderr,"Errore nell'apertura del file %s!\n",file_espressioni);
    exit(EXIT_FAILURE);
  }

  /* Legge il file una riga alla volta */
  while (fscanf(fp,"%[^\n]\n",Espressione) == 1)
  {
    /* Carica l'espressione corrente dalla stringa nell'albero */
    T = ConverteEspressione(Espressione);

    /* Calcola il valore dell'espressione rappresentata dall'albero */
    v = CalcolaValore(T);
    printf("valore     = %f\n",v);

    /* Stampa in notazione infissa l'espressione rappresentata dall'albero */
    StampaEspressione(T);
    printf("\n");

    /* Calcola la profondita' dell'albero */
    //p = Profondita(T);
    //printf("profondita = %d\n",p);

    /* Distrugge l'albero */
    distruggealbero(&T);
    printf("\n");
  }

  fclose(fp);
}


/* Carica l'espressione corrente dalla stringa nell'albero */
albero ConverteEspressione (char *Espressione)
{
  char s[ROW_LENGTH]; /* stringa che codifica l'operatore */
  char s_op1[ROW_LENGTH]; /* stringa che codifica l'operando 1 */
  char s_op2[ROW_LENGTH]; /* stringa che codifica l'operando 2 */
  Operatore op;
  Operando val;
  albero T, Ts, Td;
  int ie, is;
  int cont;
  
  
  /* Caso base: se l'espressione e' un numero reale, 
     restituisce un albero con il numero in radice e due sottoalberi vuoti */
  if (sscanf(Espressione,"%lf",&val) == 1)
    T = costruiscealbero(NO_OP,val,EMPTY_TREE,EMPTY_TREE);
  else
  {   
    /* altrimenti scompone l'espressione nei suoi tre componenti:
       l'operatore e i due operandi */
    for (ie = 0; Espressione[ie] != '('; ie++)
      s[ie] = Espressione[ie];
    s[ie] = '\0';
    
   
    ie++;
    is = 0;
    cont = 0;
    while ( (Espressione[ie] != ',') || (cont != 0) )
    {
      s_op1[is] = Espressione[ie];
      
      if (Espressione[ie] == '(')
        cont++;
      else if (Espressione[ie] == ')')
        cont--;
      
      ie++;
      is++;
    }   
    s_op1[is] = '\0';
 

    ie++;
    is = 0;
    cont = 0;
    while ( (Espressione[ie] != ')') || (cont != 0) )
    {
      s_op2[is] = Espressione[ie];
      
      if (Espressione[ie] == '(')
        cont++;
      else if (Espressione[ie] == ')')
        cont--;
      
      ie++;
      is++;
    }   
    s_op2[is] = '\0';

      
    /* Trasforma l'operatore come stringa nel carattere corrispondente */
    if (strcmp(s,"somma") == 0)
      op = '+';
    else if (strcmp(s,"diff") == 0)
      op = '-';
    else if (strcmp(s,"prod") == 0)
      op = '*';
    else if (strcmp(s,"div") == 0)
      op = '/';
 
    /* Trasforma l'operando 1 in un albero */
    Ts = ConverteEspressione(s_op1);

    /* Trasforma l'operando 2 in un albero */
    Td = ConverteEspressione(s_op2);

    /* Restituisce l'albero composto dall'operatore e dai due operandi */
    T = costruiscealbero(op,NO_VAL,Ts,Td);
  }

  return T;
}


/* Calcola il valore dell'espressione rappresentata dall'albero */
double CalcolaValore (albero T)
{
  Operatore op;
  Operando  val, vals, vald;

  /* Se l'albero e' un'espressione numerica, restituisce il valore */
  leggenodo(T,radice(T),&op,&val);
  if (op == NO_OP)
    return val;
  else
  {
  /* altrimenti, calcola il valore dei sottoalberi e li combina con l'operatore */
    vals = CalcolaValore(figliosinistro(T,radice(T)));
    vald = CalcolaValore(figliodestro(T,radice(T)));
    if (op == '+')
      return vals+vald;
    else if (op == '-')
      return vals-vald;
    else if (op == '*')
      return vals*vald;
    else if (op == '/')
      return vals/vald;
  }
}


/* Stampa in notazione infissa l'espressione rappresentata dall'albero */
void StampaEspressione (albero T)
{
  Operatore op;
  Operando  val;

  /* Se l'albero e' un'espressione numerica, stampa il valore */
  leggenodo(T,radice(T),&op,&val);
  if (op == NO_OP)
    printf(" %lf ",val);
  else
  {
  /* altrimenti, stampa fra parentesi il sottoalbero sinistro,
    poi l'operatore, infine il sottoalbero destro */
    printf(" ( ");
    StampaEspressione(figliosinistro(T,radice(T)));
    printf(" %c ",op);
    StampaEspressione(figliodestro(T,radice(T)));    
    printf(" ) ");
  }

}


/* Calcola la profondita' dell'albero */
int Profondita (albero T)
{
}
